# Microeconometrics
# CLASS 6a
# Endogeneity (choice models)
# Wiktor Budzinski

rm(list=ls())
# ctrl+l

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 6")    
load("EndoChoice.RData")

library(apollo)
library(tidyr)

################################################# Exercise 1 ########################################
# Run MNL model and calculate WTP for the wine being produced by the small, family-sized vineyard

# Output settings
modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE
# Setting seed

apollo_initialise()
apollo_control = list(modelName  ="MNL",modelDescr ="MNL with endogeneity",
                      indivID    ="id", outputDirectory = "output")
# SPECIFY COEFFICIENTS
# True: 0.7, 0.3, 0.5, -0.04
apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1
  V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2
  V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mnl = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnl, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(mnl,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))

################################################# Exercise 1 ########################################
# Use control function approach to correct for the endogeneity of price
  # Use information on the promotion of wine as an instrumental variable

data <-pivot_longer(data = database, cols = starts_with("Price"), names_to = "Price", names_prefix = "Price")
data_long <- data.frame(matrix(ncol = 1, nrow = 2400))
data_long$Price <- data$value
data <-pivot_longer(data = database, cols = starts_with("Grape"), names_to = "Grape", names_prefix = "Grape")
data_long$Grape <- data$value
data <-pivot_longer(data = database, cols = starts_with("Promo"), names_to = "Promo", names_prefix = "Promo")
data_long$Promo <- data$value
data <-pivot_longer(data = database, cols = starts_with("Fam"), names_to = "Fam", names_prefix = "Fam")
data_long$Fam <- data$value
data_long$alt <- data$Fam
data_long$id <- data$id

data_long$logPrice <-log(data_long$Price)

Eq <- logPrice ~ Promo + Fam + factor(Grape)
cf <- lm(Eq, data=data_long)
summary(cf)

# data_long$cf <- rstandard(cf) # residuals
data_long$cf <- cf$residuals
cfvar <- data.frame(pivot_wider(data = data_long, id_cols = id, names_from = alt, 
                        values_from = c("cf")))
database$cf1 <-cfvar[,2]
database$cf2 <-cfvar[,3]
database$cf3 <-cfvar[,4]
plot(database$Price1, database$cf1)
## Running mnl again

apollo_control = list(modelName  ="MNLcf",modelDescr ="MNL with control function",
                      indivID    ="id", outputDirectory = "output")
# SPECIFY COEFFICIENTS
# True: 0.7, 0.3, 0.5, -0.04
apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, betacf = 0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + betacf*cf1
  V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + betacf*cf2
  V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + betacf*cf3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mnlcf = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mnlcf, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(mnlcf,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))

################################################# Exercise 2 ########################################
# Use MIS to account for the endogeneity of price variable

data <-pivot_longer(data = database, cols = starts_with("Rating1"), names_to = "Rating1", names_prefix = "Rating1")
data_long$Rating1 <- data$value
data <-pivot_longer(data = database, cols = starts_with("Rating2"), names_to = "Rating2", names_prefix = "Rating2")
data_long$Rating2 <- data$value


# Simply adding the rating
apollo_control = list(modelName  ="MIS1",modelDescr ="MNL with MIS1",
                      indivID    ="id", outputDirectory = "output")
apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, beta_rat = 0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()

# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + beta_rat*Rating1_1
  V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + beta_rat*Rating1_2
  V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + beta_rat*Rating1_3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mis1 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mis1, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(mis1,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))

# Using the second rating in CF

Eq <- Rating1 ~ Rating2 + Fam + factor(Grape) + Price
mis <- lm(Eq, data=data_long)
summary(mis)

data_long$mis <- mis$residuals
misvar <- data.frame(pivot_wider(data = data_long, id_cols = id, names_from = alt, 
                                values_from = c("mis")))
database$misx1 <-misvar[,2]
database$misx2 <-misvar[,3]
database$misx3 <-misvar[,4]

apollo_control = list(modelName  ="MIS2",modelDescr ="MNL with MIS",
                      indivID    ="id", outputDirectory = "output")
apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, beta_rat = 0.1, beta_mis = 0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()


# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + beta_rat*Rating1_1 + beta_mis*misx1
  V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + beta_rat*Rating1_2 + beta_mis*misx2
  V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + beta_rat*Rating1_3 + beta_mis*misx3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mis2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mis2, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(mis2,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))

################################################# Exercise 2 ########################################
# Use a hybrid choice model specification to account for the endogeneity of price variable


rm(apollo_inputs) # remove inputs to not "confuse" Apollo
apollo_control = list(modelName  ="HCM",modelDescr ="Hybrid choice model - wine choice",
                      indivID    ="id", mixing = TRUE, nCores = 4, outputDirectory = "output")

apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, 
                beta_q = 0.2, 
                a_fam = 0.01, a_grape1 = 0.01, a_grape2 = -0.1, a_price = 0.03, # explanatory variables of EQ
                mu_rating1 = 0.2, lambda_rating1 = 1, sigma_rating1 = -0.1,
                mu_rating2 = 0.2, lambda_rating2 = 1, sigma_rating2 = -0.1)

#apollo_fixed = c("lambda_rating1")
apollo_fixed = c()

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 100,
  interUnifDraws = c(),
  interNormDraws = c('eta1', 'eta2', 'eta3'),
  
  intraDrawsType="",
  intraNDraws=0,          
  intraUnifDraws=c(),     
  intraNormDraws=c() 
)

apollo_randCoeff=function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["EQ1"]] = a_fam*(Fam1==2) + a_grape1*(Grape1 == 2) + a_grape2*(Grape1 == 3) + a_price*Price1 + eta1
  randcoeff[["EQ2"]] = a_fam*(Fam2==2) + a_grape1*(Grape2 == 2) + a_grape2*(Grape2 == 3) + a_price*Price2 + eta2
  randcoeff[["EQ3"]] = a_fam*(Fam3==2) + a_grape1*(Grape3 == 2) + a_grape2*(Grape3 == 3) + a_price*Price3 + eta3
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### Likelihood of indicators
  normalDensity_settings1 = list(outcomeNormal = Rating1_1, 
                                 xNormal       = lambda_rating1*EQ1, 
                                 mu            = mu_rating1, 
                                 sigma         = exp(sigma_rating1), 
                                 componentName = "indic_rating11")
  normalDensity_settings2 = list(outcomeNormal = Rating1_2, 
                                 xNormal       = lambda_rating1*EQ2, 
                                 mu            = mu_rating1, 
                                 sigma         = exp(sigma_rating1), 
                                 componentName = "indic_rating12")
  normalDensity_settings3 = list(outcomeNormal = Rating1_3, 
                                 xNormal       = lambda_rating1*EQ3, 
                                 mu            = mu_rating1, 
                                 sigma         = exp(sigma_rating1), 
                                 componentName = "indic_rating13")
  normalDensity_settings4 = list(outcomeNormal = Rating2_1, 
                                 xNormal       = lambda_rating2*EQ1, 
                                 mu            = mu_rating2, 
                                 sigma         = exp(sigma_rating2), 
                                 componentName = "indic_rating21")
  normalDensity_settings5 = list(outcomeNormal = Rating2_2, 
                                 xNormal       = lambda_rating2*EQ2, 
                                 mu            = mu_rating2, 
                                 sigma         = exp(sigma_rating2), 
                                 componentName = "indic_rating22")
  normalDensity_settings6 = list(outcomeNormal = Rating2_3, 
                                 xNormal       = lambda_rating2*EQ3, 
                                 mu            = mu_rating2, 
                                 sigma         = exp(sigma_rating2), 
                                 componentName = "indic_rating23") 
  
  P[["indic_rating1"]]     = apollo_normalDensity(normalDensity_settings1, functionality)
  P[["indic_rating2"]] = apollo_normalDensity(normalDensity_settings2, functionality)
  P[["indic_rating3"]]      = apollo_normalDensity(normalDensity_settings3, functionality)
  P[["indic_rating4"]]      = apollo_normalDensity(normalDensity_settings4, functionality)
  P[["indic_rating5"]]      = apollo_normalDensity(normalDensity_settings5, functionality)
  P[["indic_rating6"]]      = apollo_normalDensity(normalDensity_settings6, functionality)
  
  ### Likelihood of choices
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_q*EQ1 + beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1
  V[['alt2']]  = beta_q*EQ2 + beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2
  V[['alt3']]  = beta_q*EQ3 + beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities for MNL model component
  P[["choice"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Likelihood of the whole model
  P = apollo_combineModels(P, apollo_inputs, functionality)
  
  ### Average across intra-individual draws
#   P = apollo_avgIntraDraws(P, apollo_inputs, functionality)
  
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

hcm = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(hcm, modelOutput_settings)

apollo_deltaMethod(hcm,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))
