# Microeconometrics
# CLASS 6b
# Endogeneity (MIS vs. HCM)
# Wiktor Budzinski

rm(list=ls())
# ctrl+l

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2024\\Class 6")    
load("EndoChoiceOrd.RData")

library(apollo)
library(tidyr)


# Output settings
modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE


################################################# Exercise 3 ########################################
#  Compare the WTP estimates from MIS and SEM using the EndoChoiceOrd.Rdata 
# First MIS

data <-pivot_longer(data = database, cols = starts_with("Price"), names_to = "Price", names_prefix = "Price")
data_long <- data.frame(matrix(ncol = 1, nrow = 2400))
data_long$Price <- data$value
data <-pivot_longer(data = database, cols = starts_with("Grape"), names_to = "Grape", names_prefix = "Grape")
data_long$Grape <- data$value
data <-pivot_longer(data = database, cols = starts_with("Fam"), names_to = "Fam", names_prefix = "Fam")
data_long$alt <- data$Fam
data_long$Fam <- data$value
data_long$id <- data$id
data <-pivot_longer(data = database, cols = starts_with("Rating1"), names_to = "Rating1", names_prefix = "Rating1")
data_long$Rating1 <- data$value

hist(data_long$Rating1)

data <-pivot_longer(data = database, cols = starts_with("Rating2"), names_to = "Rating2", names_prefix = "Rating2")
data_long$Rating2 <- data$value


Eq <- Rating1 ~ Rating2 + Fam + factor(Grape) + Price
mis <- lm(Eq, data=data_long)
summary(mis)

data_long$mis <- mis$residuals
misvar <- data.frame(pivot_wider(data = data_long, id_cols = id, names_from = alt, 
                                 values_from = c("mis")))
database$misx1 <-misvar[,2]
database$misx2 <-misvar[,3]
database$misx3 <-misvar[,4]

apollo_control = list(modelName  ="MIS2",modelDescr ="MNL with MIS",
                      indivID    ="id", outputDirectory = "output")
apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, beta_rat = 0.1, beta_mis = 0.1)
apollo_fixed = c()
apollo_inputs = apollo_validateInputs()


# DEFINE MNL
apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + beta_rat*Rating1_1 + beta_mis*misx1
  V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + beta_rat*Rating1_2 + beta_mis*misx2
  V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + beta_rat*Rating1_3 + beta_mis*misx3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities using MNL model
  P[["model"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# Estimation                                      
mis2_ord = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(mis2_ord, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(mis2_ord,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))


################################################# Exercise 3 ########################################
# Now Hybrid choice

rm(apollo_inputs) # remove inputs to not "confuse" Apollo
apollo_control = list(modelName  ="HCM",modelDescr ="Hybrid choice model - wine choice",
                      indivID    ="id", mixing = TRUE, nCores = 4, outputDirectory = "output")

apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, 
                beta_q = 0.2, 
                a_fam = 0.01, a_grape1 = 0.01, a_grape2 = -0.1, a_price = 0.03, # explanatory variables of EQ
                lambda_rating1 = 1, tau_11 = -1, tau_12 = -0.5, tau_13 = 0.5, tau_14 = 1.5,
                lambda_rating2 = 1, tau_21 = -1, tau_22 = -0.5, tau_23 = 0.5, tau_24 = 1.5)

apollo_fixed = c("lambda_rating1")

apollo_draws = list(
  interDrawsType = "halton",
  interNDraws    = 100,
  interUnifDraws = c(),
  interNormDraws = c('eta1', 'eta2', 'eta3'),
  
  intraDrawsType="",
  intraNDraws=0,          
  intraUnifDraws=c(),     
  intraNormDraws=c() 
)

apollo_randCoeff=function(apollo_beta, apollo_inputs){
  randcoeff = list()
  randcoeff[["EQ1"]] = a_fam*(Fam1==2) + a_grape1*(Grape1 == 2) + a_grape2*(Grape1 == 3) + a_price*Price1 + eta1
  randcoeff[["EQ2"]] = a_fam*(Fam2==2) + a_grape1*(Grape2 == 2) + a_grape2*(Grape2 == 3) + a_price*Price2 + eta2
  randcoeff[["EQ3"]] = a_fam*(Fam3==2) + a_grape1*(Grape3 == 2) + a_grape2*(Grape3 == 3) + a_price*Price3 + eta3
  return(randcoeff)
}

apollo_inputs = apollo_validateInputs()

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
  
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))
  
  ### Create list of probabilities P
  P = list()
  
  ### Likelihood of indicators

  ol_settings1 = list(outcomeOrdered = Rating1_1, 
                                 V       = lambda_rating1*EQ1, 
                                 tau            = list(tau_11, tau_12, tau_13, tau_14), 
                                 componentName = "indic_rating11")
  ol_settings2 = list(outcomeOrdered = Rating1_2, 
                                 V       = lambda_rating1*EQ2, 
                                 tau            = list(tau_11, tau_12, tau_13, tau_14), 
                                 componentName = "indic_rating12")
  ol_settings3 = list(outcomeOrdered = Rating1_3, 
                                 V       = lambda_rating1*EQ3, 
                                 tau            = list(tau_11, tau_12, tau_13, tau_14), 
                                 componentName = "indic_rating13")
  ol_settings4 = list(outcomeOrdered = Rating2_1, 
                                 V      = lambda_rating2*EQ1, 
                                 tau            = list(tau_21, tau_22, tau_23, tau_24), 
                                 componentName = "indic_rating21")
  ol_settings5 = list(outcomeOrdered = Rating2_2, 
                                 V       = lambda_rating2*EQ2, 
                                 tau            = list(tau_21, tau_22, tau_23, tau_24),
                                 componentName = "indic_rating22")
  ol_settings6 = list(outcomeOrdered = Rating2_3, 
                                 V      = lambda_rating2*EQ3, 
                                 tau            = list(tau_21, tau_22, tau_23, tau_24),
                                 componentName = "indic_rating23") 
  
  P[["indic_rating1"]]     = apollo_ol(ol_settings1, functionality)
  P[["indic_rating2"]]     = apollo_ol(ol_settings2, functionality)
  P[["indic_rating3"]]      = apollo_ol(ol_settings3, functionality)
  P[["indic_rating4"]]      = apollo_ol(ol_settings4, functionality)
  P[["indic_rating5"]]      = apollo_ol(ol_settings5, functionality)
  P[["indic_rating6"]]      = apollo_ol(ol_settings6, functionality)
  
  ### Likelihood of choices
  ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
  V = list()
  V[['alt1']]  = beta_q*EQ1 + beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1
  V[['alt2']]  = beta_q*EQ2 + beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2
  V[['alt3']]  = beta_q*EQ3 + beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3
  
  ### Define settings for MNL model component
  mnl_settings = list(
    alternatives = c(alt1=1, alt2=2, alt3=3),
    avail        = list(alt1=1, alt2=1, alt3=1),
    choiceVar    = choice,
    V = V
  )
  
  ### Compute probabilities for MNL model component
  P[["choice"]] = apollo_mnl(mnl_settings, functionality)
  
  ### Likelihood of the whole model
  P = apollo_combineModels(P, apollo_inputs, functionality)
  
  ### Average across intra-individual draws
  #   P = apollo_avgIntraDraws(P, apollo_inputs, functionality)
  
  P = apollo_avgInterDraws(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

hcm_ord = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
apollo_modelOutput(hcm_ord, modelOutput_settings)

# True value: 17.5
apollo_deltaMethod(hcm_ord,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))
