# Microeconometrics
# CLASS 6c
# Endogeneity - bootstrap
# Wiktor Budzinski

rm(list=ls())
# ctrl+l

setwd("C:\\Users\\budzi\\Documents\\ME QPE\\2023\\Class 6")    
load("EndoChoice.RData")

library(apollo)
library(tidyr)
library(matrixStats)

################################################# Exercise 4 ########################################
# Compare the distribution of WTP obtained in exercise 2 from the control function and MIS, with the one obtained from bootstrap
  # First control function with bootstrap

# Output settings
modelOutput_settings=list()
modelOutput_settings[['printPVal']] <- 2 # Print p-values
modelOutput_settings[['printClassical']] <- FALSE
# Setting seed

apollo_initialise()

################################################# Exercise 2 ########################################
# Use  a control function approach to correct for endogeneity
  # Using bootstrap to obtain correct standard errors

NSim <- 250 # Should be higher 
set.seed(1000001) 
dataMain <-database
WTP_sim <- matrix(, nrow = NSim, ncol = 1)
for(i in 1:NSim){
  ID_i <- sample(dataMain$id, 800, replace = TRUE)
  database <- dataMain[ID_i,]
  database$id_new <- 1:800
  data <-pivot_longer(data = database, cols = starts_with("Price"), names_to = "Price", names_prefix = "Price")
  data_long <- data.frame(matrix(ncol = 1, nrow = 2400))
  data_long$Price <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Grape"), names_to = "Grape", names_prefix = "Grape")
  data_long$Grape <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Promo"), names_to = "Promo", names_prefix = "Promo")
  data_long$Promo <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Fam"), names_to = "Fam", names_prefix = "Fam")
  data_long$Fam <- data$value
  data_long$alt <- data$Fam
  data_long$id_new <- data$id_new
  data_long$logPrice <-log(data_long$Price)
  Eq <- logPrice ~ Promo + Fam + factor(Grape)
  cf <- lm(Eq, data=data_long)
  data_long$cf <- cf$residuals
  
  cfvar <- data.frame(pivot_wider(data = data_long, id_cols = id_new, names_from = alt, 
                                  values_from = c("cf")))
  database$cf1 <-cfvar[,2]
  database$cf2 <-cfvar[,3]
  database$cf3 <-cfvar[,4]
  
  apollo_control = list(modelName  ="MNLcf",modelDescr ="MNL with control function",
                        indivID    ="id_new", outputDirectory = "output")
  apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, betacf = 0.1)
  apollo_fixed = c()
  apollo_inputs = apollo_validateInputs()
  
  apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
    
    ### Attach inputs and detach after function exit
    apollo_attach(apollo_beta, apollo_inputs)
    on.exit(apollo_detach(apollo_beta, apollo_inputs))
    
    ### Create list of probabilities P
    P = list()
    
    ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
    V = list()
    V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + betacf*cf1
    V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + betacf*cf2
    V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + betacf*cf3
    
    ### Define settings for MNL model component
    mnl_settings = list(
      alternatives = c(alt1=1, alt2=2, alt3=3),
      avail        = list(alt1=1, alt2=1, alt3=1),
      choiceVar    = choice,
      V = V
    )
    
    ### Compute probabilities using MNL model
    P[["model"]] = apollo_mnl(mnl_settings, functionality)
    
    ### Prepare and return outputs of function
    P = apollo_prepareProb(P, apollo_inputs, functionality)
    return(P)
  }
  # Estimation                                      
  mnlcf = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
  # True value: 17.5
  delta <- apollo_deltaMethod(mnlcf,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))
  WTP_sim[i,] <- delta[1]
}

hist(WTP_sim)
colMeans(WTP_sim)
apply(WTP_sim, 2, sd)
colQuantiles(WTP_sim, probs = 0.05)
colQuantiles(WTP_sim, probs = 0.95)

################################################# Exercise 4 ########################################
# Compare the distribution of WTP obtained in exercise 2 from the control function and MIS, with the one obtained from bootstrap
  # Now MIS

NSim <- 250 # Should be higher 
set.seed(1000001) 
dataMain <-database
WTP_sim2 <- matrix(, nrow = NSim, ncol = 1)


for(i in 1:NSim){
  ID_i <- sample(dataMain$id, 800, replace = TRUE)
  database <- dataMain[ID_i,]
  database$id_new <- 1:800
  
  data <-pivot_longer(data = database, cols = starts_with("Price"), names_to = "Price", names_prefix = "Price")
  data_long <- data.frame(matrix(ncol = 1, nrow = 2400))
  data_long$Price <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Grape"), names_to = "Grape", names_prefix = "Grape")
  data_long$Grape <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Promo"), names_to = "Promo", names_prefix = "Promo")
  data_long$Promo <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Fam"), names_to = "Fam", names_prefix = "Fam")
  data_long$Fam <- data$value
  data_long$alt <- data$Fam
  data_long$id_new <- data$id_new
  
  data <-pivot_longer(data = database, cols = starts_with("Rating1"), names_to = "Rating1", names_prefix = "Rating1")
  data_long$Rating1 <- data$value
  data <-pivot_longer(data = database, cols = starts_with("Rating2"), names_to = "Rating2", names_prefix = "Rating2")
  data_long$Rating2 <- data$value
  
  Eq <- Rating1 ~ Rating2 + Fam + factor(Grape) + Price
  mis <- lm(Eq, data=data_long)
  summary(mis)
  
  data_long$mis <- mis$residuals
  misvar <- data.frame(pivot_wider(data = data_long, id_cols = id_new, names_from = alt, 
                                   values_from = c("mis")))
  database$misx1 <-misvar[,2]
  database$misx2 <-misvar[,3]
  database$misx3 <-misvar[,4]
  
  apollo_control = list(modelName  ="MIS2",modelDescr ="MNL with MIS",
                        indivID    ="id_new", outputDirectory = "output")
  apollo_beta = c(beta_fam = 0.1, beta_grape1 = 0.1, beta_grape2 = 0.1, beta_price = -0.1, beta_rat = 0.1, beta_mis = 0.1)
  apollo_fixed = c()
  apollo_inputs = apollo_validateInputs()
  
  
  # DEFINE MNL
  apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
    
    ### Attach inputs and detach after function exit
    apollo_attach(apollo_beta, apollo_inputs)
    on.exit(apollo_detach(apollo_beta, apollo_inputs))
    
    ### Create list of probabilities P
    P = list()
    
    ### List of utilities: these must use the same names as in mnl_settings, order is irrelevant
    V = list()
    V[['alt1']]  = beta_fam*(Fam1==2) + beta_grape1*(Grape1 == 2) + beta_grape2*(Grape1 == 3) + beta_price*Price1 + beta_rat*Rating1_1 + beta_mis*misx1
    V[['alt2']]  = beta_fam*(Fam2==2) + beta_grape1*(Grape2 == 2) + beta_grape2*(Grape2 == 3) + beta_price*Price2 + beta_rat*Rating1_2 + beta_mis*misx2
    V[['alt3']]  = beta_fam*(Fam3==2) + beta_grape1*(Grape3 == 2) + beta_grape2*(Grape3== 3) + beta_price*Price3 + beta_rat*Rating1_3 + beta_mis*misx3
    
    ### Define settings for MNL model component
    mnl_settings = list(
      alternatives = c(alt1=1, alt2=2, alt3=3),
      avail        = list(alt1=1, alt2=1, alt3=1),
      choiceVar    = choice,
      V = V
    )
    
    ### Compute probabilities using MNL model
    P[["model"]] = apollo_mnl(mnl_settings, functionality)
    
    ### Prepare and return outputs of function
    P = apollo_prepareProb(P, apollo_inputs, functionality)
    return(P)
  }
  
  # Estimation                                      
  mis2 = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)
  delta <- apollo_deltaMethod(mis2,deltaMethod_settings = list(operation="ratio",parName1="beta_fam", parName2="beta_price",multPar1=-1))
  WTP_sim2[i,] <- delta[1]
}

hist(WTP_sim2)
colMeans(WTP_sim2)
apply(WTP_sim2, 2, sd)
colQuantiles(WTP_sim2, probs = 0.05)
colQuantiles(WTP_sim2, probs = 0.95)